namespace :git do

  set :git_environmental_variables, ->() {
    {
      git_askpass: "/bin/echo",
      git_ssh:     "#{fetch(:tmp_dir)}/#{fetch(:application)}/git-ssh.sh"
    }
  }

  desc 'Upload the git wrapper script, this script guarantees that we can script git without getting an interactive prompt'
  task :wrapper do
    on release_roles :all do
      execute :mkdir, "-p", "#{fetch(:tmp_dir)}/#{fetch(:application)}/"
      upload! StringIO.new("#!/bin/sh -e\nexec /usr/bin/ssh -o PasswordAuthentication=no -o StrictHostKeyChecking=no \"$@\"\n"), "#{fetch(:tmp_dir)}/#{fetch(:application)}/git-ssh.sh"
      execute :chmod, "+x", "#{fetch(:tmp_dir)}/#{fetch(:application)}/git-ssh.sh"
    end
  end

  desc 'Check that the repository is reachable'
  task check: :'git:wrapper' do
    fetch(:branch)
    on release_roles :all do
      with fetch(:git_environmental_variables) do
        exit 1 unless test :git, :'ls-remote', repo_url
      end
    end
  end

  desc 'Clone the repo to the cache'
  task clone: :'git:wrapper' do
    on release_roles :all do
      if test " [ -f #{repo_path}/HEAD ] "
        info t(:mirror_exists, at: repo_path)
      else
        within deploy_path do
          with fetch(:git_environmental_variables) do
            execute :git, :clone, '--mirror', repo_url, repo_path
          end
        end
      end
    end
  end

  desc 'Update the repo mirror to reflect the origin state'
  task update: :'git:clone' do
    on release_roles :all do
      within repo_path do
        capturing_revisions do
          execute :git, :remote, :update
        end
      end
    end
  end

  desc 'Copy repo to releases'
  task create_release: :'git:update' do
    invoke "git:create_release_via_#{fetch(:git_strategy, :archive)}"
  end

  task :create_release_via_clone do
    on roles :all do
      with fetch(:git_environmental_variables) do
        within repo_path do
          execute :git, :clone, '--branch', fetch(:branch),               \
            '--depth 1',                                                  \
            '--recursive',                                                \
            '--no-hardlinks',                                             \
            repo_path, release_path
        end
      end
    end
  end

  task :create_release_via_archive do
    on release_roles :all do
      with fetch(:git_environmental_variables) do
        within repo_path do
          execute :mkdir, '-p', release_path
          execute :git, :archive, fetch(:branch), '| tar -x -C', release_path
        end
      end
    end
  end
end

